/*
FILE
	dashboard.c
	$Id: dashboard.c 660 2008-12-18 18:48:48Z hus $
PURPOSE
	Provide the functions for the idnsAdmin Dashboard.
AUTHOR
	(C) Hugo Urquiza 2006-2009 for Unixservice.
*/

#include "interface.h"

char *cGetSystemHealth(void);
static unsigned uTimeStep=0;

void SetTimeStep(void);
#ifdef EXPERIMENTAL
unsigned uNamedCheckConf(void);
unsigned uHDStatus(void);
#endif

void SetTimeStep(void)
{
	//This function sets the module global uTimeStep variable
	//That variable will be used at the Dashboard tables
	//for specifying how long we should look back to get tLog data.
	//We call this function once, when htmlDashBoard() is called.
	unsigned uWeekDay=0;
	MYSQL_RES *res;
	MYSQL_ROW field;

	sprintf(gcQuery,"SELECT WEEKDAY(NOW())");
	mysql_query(&gMysql,gcQuery);
	if(mysql_errno(&gMysql))
		htmlPlainTextError(mysql_error(&gMysql));
	
	res=mysql_store_result(&gMysql);

	//The query always returns a row
	field=mysql_fetch_row(res);
	sscanf(field[0],"%u",&uWeekDay);

	//Monday is day 0
	if(uWeekDay<=2)
		uTimeStep=777600; //9 days
	else
		uTimeStep=604800; //7 days

}//void SetTimeStep(void)


void htmlDashBoard(void)
{
	SetTimeStep();
	gcCustomer[0]=0;
	gcZone[0]=0;
	cuView[0]=0;
	uResource=0;
	guContact=0;
	sys_SetSessionCookie();
	
	htmlHeader("iDNS System","Header");
	htmlDashBoardPage("","DashBoard.Body");
	htmlFooter("Footer");

}//void htmlDashBoard(void)


void htmlDashBoardPage(char *cTitle, char *cTemplateName)
{
	if(cTemplateName[0])
	{
        	MYSQL_RES *res;
	        MYSQL_ROW field;

		TemplateSelect(cTemplateName);
		res=mysql_store_result(&gMysql);
		if((field=mysql_fetch_row(res)))
		{
			struct t_template template;
			char cuResource[16]={""};

			sprintf(cuResource,"%u",uResource);

			template.cpName[0]="cTitle";
			template.cpValue[0]=cTitle;
			
			template.cpName[1]="cCGI";
			template.cpValue[1]="idnsAdmin.cgi";
			
			template.cpName[2]="cMessage";
			template.cpValue[2]=gcMessage;

			template.cpName[3]="gcLogin";
			template.cpValue[3]=gcLogin;

			template.cpName[4]="gcName";
			template.cpValue[4]=gcName;

			template.cpName[5]="gcOrgName";
			template.cpValue[5]=gcOrgName;

			template.cpName[6]="cUserLevel";
			template.cpValue[6]=(char *)cUserLevel(guPermLevel);

			template.cpName[7]="gcHost";
			template.cpValue[7]=gcHost;
			
			template.cpName[8]="cSystemHealth";
			template.cpValue[8]=cGetSystemHealth();

			template.cpName[9]="cZone";
			template.cpValue[9]=gcZone;

			template.cpName[10]="cCustomer";
			template.cpValue[10]=gcCustomer;

			template.cpName[11]="uView";
			template.cpValue[11]=cuView;

			template.cpName[12]="uResource";
			template.cpValue[12]=cuResource;

			template.cpName[13]="";

			printf("\n<!-- Start htmlDashBoardPage(%s) -->\n",cTemplateName); 
			Template(field[0], &template, stdout);
			printf("\n<!-- End htmlDashBoardPage(%s) -->\n",cTemplateName); 
		}
		else
		{
			printf("<hr>");
			printf("<center><font size=1>%s</font>\n",cTemplateName);
		}
		mysql_free_result(res);
	}

}//void htmlDashBoardPage(char *cTitle, char *cTemplateName)


char *cGetSystemHealth(void)
{
	//
	//This function will provide a quick status report with colors
	//based on the informattion displayed at the right panel of the
	//dashboard
	//
	//Green: no jobs pending, no bind errors
	//Yellow: more than 4 jobs pending, no bind errors
	//Red: more than 4 jobs pending, bind errors
	//
	//The bind erros or not condition is within the last 24 hrs
	//
	MYSQL_RES *res;
	MYSQL_ROW field;
	unsigned uJobCount=0;
	unsigned uErrorCount=0;
	unsigned uCriticalCount=0; //critial errors are those generated by dns query timeouts to any of the nameservers in the cluster.
	static char cSystemHealth[100]={""};
	
	sprintf(gcQuery,"SELECT COUNT(uLog) FROM tLog WHERE uLogType=5 AND ((uCreatedDate >= UNIX_TIMESTAMP(NOW())-86400) OR (uModDate>= UNIX_TIMESTAMP(NOW())-86400))");
	mysql_query(&gMysql,gcQuery);
	if(mysql_errno(&gMysql))
		htmlPlainTextError(mysql_error(&gMysql));
	res=mysql_store_result(&gMysql);
	if((field=mysql_fetch_row(res)))
		sscanf(field[0],"%u",&uErrorCount);

	mysql_free_result(res);

	sprintf(gcQuery,"SELECT COUNT(uJob) FROM tJob WHERE cTargetServer!='graphics.server'");
	mysql_query(&gMysql,gcQuery);
	if(mysql_errno(&gMysql))
		htmlPlainTextError(mysql_error(&gMysql));
	res=mysql_store_result(&gMysql);
	if((field=mysql_fetch_row(res)))
		sscanf(field[0],"%u",&uJobCount);

	mysql_free_result(res);
	
	sprintf(gcQuery,"SELECT COUNT(uLog) FROM tLog WHERE uLogType=4 AND cMessage LIKE '%%dnsMonitor%%' AND ((uCreatedDate >= UNIX_TIMESTAMP(NOW())-3600) OR (uModDate>= UNIX_TIMESTAMP(NOW())-3600))");
	mysql_query(&gMysql,gcQuery);
	if(mysql_errno(&gMysql))
		htmlPlainTextError(mysql_error(&gMysql));
	res=mysql_store_result(&gMysql);
	if((field=mysql_fetch_row(res)))
		sscanf(field[0],"%u",&uCriticalCount);
	
	if(uCriticalCount)
	{
		sprintf(cSystemHealth,"<p style='color: red;'><b>Critical (connection timeout to one or more nameservers) [%u]</b></p>",uCriticalCount);
		return(cSystemHealth); //this condition surpasses all evaluated below.
	}
#ifdef EXPERIMENTAL
	if((uNamedCheckConf()))
	{
		sprintf(cSystemHealth,"<p style='color: red;'><b>Critical (BIND9 conf has errors)</b></p>");
		return(cSystemHealth); //this condition surpasses all evaluated below.
	}
#endif
	if(!uJobCount && !uErrorCount)
		sprintf(cSystemHealth,"<p style='color: green;'><b>Green</b></p>");	
	else if(uJobCount>4 && !uErrorCount)
		sprintf(cSystemHealth,"<p style='color: #DAA520;'><b>Yellow</b></p>");
	else if(!uJobCount && uErrorCount)
		sprintf(cSystemHealth,"<p style='color: #DAA520;'><b>Yellow</b></p>");
	else if(uJobCount>4 && uErrorCount)
		sprintf(cSystemHealth,"<p style='color: red;'><b>Red</b></p>");
	else if(1)
		//Unknown condition ?
		sprintf(cSystemHealth,"<p style='color: red;'><b>Critical (unknown)</b></p>");

	return(cSystemHealth);
		
	
}//char *cGetSystemHealth(void)


void funcUsageGraph(FILE *fp)
{
	char cOptionalGraph[100]={""};
	
	
	GetConfiguration("allzone.stats",cOptionalGraph,1);

	if(!cOptionalGraph[0]) return;

	fprintf(fp,"<a title='This graph shows all BIND9 zone stats aggregated from all NS cluster nameservers.' "
		"class=inputLink href=\"#\" onClick=\"open_popup('idnsAdmin.cgi?gcPage=Glossary&cLabel=NS Cluster Usage')\">"
		"<strong>NS Cluster Usage</strong></a></td><tr><td></td><td colspan=5 align=right>");
	fprintf(fp,"<a title='Click to enlarge' href='%1$s'><img src='%1$s' border=0 height=154 width=400></a></td></tr>\n",cOptionalGraph);
	
}//void funcUsageGraph(FILE *fp)


void funcTopTenidnsOrgUsers(FILE *fp)
{
	MYSQL_RES *res;
	MYSQL_ROW field;

	sprintf(gcQuery,"SELECT tLog.cLogin,tClient.cLabel,tClient.uOwner AS uCompany,"
			"(SELECT cLabel FROM tClient WHERE uClient=uCompany),"
			"tLog.cHost,COUNT(tLog.uLog) AS uLoginCount,tClient.uClient "
			"FROM tLog,tClient,tAuthorize WHERE tLog.cLogin=tAuthorize.cLabel AND "
			"tAuthorize.uCertClient=tClient.uClient AND tLog.uLogType=8 AND "
			"tLog.uCreatedDate>(UNIX_TIMESTAMP(NOW())-%u) AND tLog.cLabel LIKE 'login%%' "
			"GROUP BY tLog.cLogin ORDER BY uLoginCount DESC LIMIT 10",uTimeStep);
	mysql_query(&gMysql,gcQuery);
	
	if(mysql_errno(&gMysql))
		htmlPlainTextError(mysql_error(&gMysql));
	res=mysql_store_result(&gMysql);
	
	while((field=mysql_fetch_row(res)))
		fprintf(fp,"<tr><td>&nbsp;</td>"
			//Contacts tab link
			"<td><a title='Click to load into the Contacts tab' class=darkLink "
			"href=\"idnsAdmin.cgi?gcPage=CustomerUser&uClient=%s&cCustomer=%s\">%s</td>"
			//Contacts tab link
			"<td><a title='Click to load into the Contacts tab' class=darkLink "
			"href=\"idnsAdmin.cgi?gcPage=CustomerUser&uClient=%s&cCustomer=%s\">%s</td>"
			//Companies tab link
			"<td><a title='Click to load this company in the Companies tab' class=darkLink "
			"href=idnsAdmin.cgi?gcPage=Customer&uClient=%s>%s</td>"
			"<td>%s</td><td align=right>%s</td></tr>\n",
			field[6]
			,field[3]
			,field[0]
			,field[6]
			,field[3]
			,field[1]
			,field[2]
			,field[3]
			,field[4]
			,field[5]);

}//void funcTopTenidnsOrgUsers(FILE *fp)

	
void funcTopTenidnsAdminUsers(FILE *fp)
{
	MYSQL_RES *res;
	MYSQL_ROW field;

	sprintf(gcQuery,"SELECT tLog.cLogin,tClient.cLabel,tClient.uOwner AS uCompany,"
			"(SELECT cLabel FROM tClient WHERE uClient=uCompany),"
			"tLog.cHost,COUNT(tLog.uLog) AS uLoginCount,tClient.uClient "
			"FROM tLog,tClient,tAuthorize WHERE tLog.cLogin=tAuthorize.cLabel AND "
			"tAuthorize.uCertClient=tClient.uClient AND tLog.uLogType=7 AND "
			"tLog.uCreatedDate>(UNIX_TIMESTAMP(NOW())-%u) AND tLog.cLabel LIKE 'login%%' "
			"GROUP BY tLog.cLogin ORDER BY uLoginCount DESC LIMIT 10",uTimeStep);
	mysql_query(&gMysql,gcQuery);
	if(mysql_errno(&gMysql))
		htmlPlainTextError(mysql_error(&gMysql));
	res=mysql_store_result(&gMysql);
	
	/*
	tLog.cLogin 0
	tClient.cLabel 1
	tClient.uOwner 2 (uCompany)
	gcCustomer 3
	"tLog.cHost 4
	uLoginCount 5
	tClient.uClient 6
	*/
	while((field=mysql_fetch_row(res)))
		fprintf(fp,"<tr><td>&nbsp;</td>"
			//Administrators tab link
			"<td><a title='Click to load into the Contacts tab' class=darkLink "
			"href=\"idnsAdmin.cgi?gcPage=CustomerUser&uClient=%1$s&cCustomer=%2$s"
			"&cZone=%3$s&uView=%4$s&uResource=%5$u\">%6$s</td>"
			//CustomerUsers tab link
			"<td><a title='Click to load into the CustomerUsers tab' class=darkLink "
			"href=\"idnsAdmin.cgi?gcPage=CustomerUser&uClient=%1$s&cCustomer=%2$s"
			"&cZone=%3$s&uView=%4$s&uResource=%5$u\">%7$s</td>"
			//Companies tab link
			"<td><a title='Click to load this company in the Companies tab' class=darkLink "
			"href=idnsAdmin.cgi?gcPage=Customer&uClient=%8$s>%2$s</td>"
			"<td>%9$s</td><td align=right>%10$s</td></tr>\n",
			field[6]
			,field[3]
			,gcZone
			,cuView
			,uResource
			,field[0]
			,field[1]
			,field[2]
			,field[4]
			,field[5]);

}//void funcTopTenidnsAdminUsers(FILE *fp)


void funcTopTenZoneMods(FILE *fp)
{
	MYSQL_RES *res;
	MYSQL_ROW field;
	//NOTE: -%u secs=7 days, if monday last 9 days
	sprintf(gcQuery,"SELECT tZone.cZone,tZone.uView,(SELECT tClient.cLabel FROM "
			"tClient WHERE tClient.uClient=tZone.uOwner),tZone.uOwner,"
			"COUNT(tLog.uLog) AS uLogCount FROM tZone,tLog,tResource "
			"WHERE tLog.cLabel='Mod' AND tLog.cTableName='tResource' AND "
			"tLog.uTablePK=tResource.uResource AND "
			"tLog.uCreatedDate>(UNIX_TIMESTAMP(NOW())-%u) AND tResource.uZone=tZone.uZone "
			"GROUP BY tZone.cZone ORDER BY uLogCount DESC LIMIT 10",uTimeStep);
	mysql_query(&gMysql,gcQuery);
	
	if(mysql_errno(&gMysql))
		htmlPlainTextError(mysql_error(&gMysql));
	res=mysql_store_result(&gMysql);
	
	while((field=mysql_fetch_row(res)))
		fprintf(fp,"<tr><td>&nbsp;</td>"
			//Zones tab link
			"<td><a title='Click to load into the Zones tab' class=darkLink "	
			"href=\"idnsAdmin.cgi?gcPage=Zone&cZone=%s&uView=%s&cCustomer=%s\">%s</a></td>"
			//Companies tab link
			"<td>&nbsp;</td><td colspan=2><a title='Click to load this company in the Companies tab' class=darkLink "
			"href=idnsAdmin.cgi?gcPage=Customer&uClient=%s>%s</td>"
			"<td align=right colspan=2>%s</td></tr>\n",
			field[0]
			,field[1]
			,field[2]
			,field[0]
			,field[3]
			,field[2]
			,field[4]);
				
}//void funcTopTenZoneMods(FILE *fp)


void funcTopTenTraffZones(FILE *fp)
{
	MYSQL_RES *res;
	MYSQL_ROW field;

	sprintf(gcQuery,"SELECT tZone.cZone,tZone.uView,(SELECT tClient.cLabel FROM "
			"tClient WHERE tClient.uClient=tZone.uOwner),tZone.uOwner,"
			"SUM(uHitCount) AS HitCount FROM tHit,tZone WHERE tHit.cZone=tZone.cZone "
			"AND (tHit.uCreatedDate>(UNIX_TIMESTAMP(NOW())-%u) OR tHit.uModDate>(UNIX_TIMESTAMP(NOW())-%u)) "
			"GROUP BY tZone.cZone ORDER BY HitCount DESC LIMIT 10",uTimeStep,uTimeStep);
			
	mysql_query(&gMysql,gcQuery);
	
	if(mysql_errno(&gMysql))
		htmlPlainTextError(mysql_error(&gMysql));
	res=mysql_store_result(&gMysql);
	
	while((field=mysql_fetch_row(res)))
		fprintf(fp,"<tr><td>&nbsp;</td>"
			//Zones tab link
			"<td><a title='Click to load this zone in the Zones tab' class=darkLink "	
			"href=\"idnsAdmin.cgi?gcPage=Zone&cZone=%s&uView=%s&cCustomer=%s\">%s</a></td>"
			//Companies tab link
			"<td>&nbsp;</td><td><a title='Click to load this company in the Companies tab' class=darkLink "
			"href=idnsAdmin.cgi?gcPage=Customer&uClient=%s>%s</td>"
			"<td align=right colspan=2>%s</td></tr>\n",
			field[0]
			,field[1]
			,field[2]
			,field[0]
			,field[3]
			,field[2]
			,field[4]);

}//void funcTopTenTraffZones(FILE *fp)

#ifdef EXPERIMENTAL
unsigned uNamedCheckConf(void)
{
	//This function runs named-checkconf to check BIND9
	//config file syntax. Returns 0 on success, 1 on failure.

	char ciDNSRoot[100]={"/usr/local/idns"};
	char cExtBindDir[100]={"/usr/sbin"};

	GetConfiguration("ciDNSRoot",ciDNSRoot,1);
	GetConfiguration("cExtBinDir",cExtBindDir,1);

	sprintf(gcQuery,"%s/named-checkconf %s/named.conf",cExtBindDir,ciDNSRoot);
	if((system(gcQuery)))
		return(1);
	return(0);

}//unsigned uNamedCheckConf(void)


unsigned uHDStatus(void)
{
	//This function runs df -k and df -i for checking HD status
	// Returns 0 on success, 1 on failure.

	return(0);
}//unsigned uHDStatus(void)
#endif
